@@ -62,16 +62,9 @@ get_levelpaths <- function(x, override_factor = NULL, status = FALSE, cores = NU
   cl <- NULL
 
   if(!is.null(cores)) {
-    if(!requireNamespace("parallel", quietly = TRUE)) {
-      stop("parallel required if using cores input")
-    }
-    if(is.numeric(cores)) {
-      cl <- parallel::makeCluster(cores)
-      stop_cluster <- TRUE
-    } else {
-      if(!"cluster" %in% class(cores)) {
-        stop("cores must be numeric or a cluster object")
-      }
+    cl <- get_cl(cores)
+    if(!inherits(cores, "cluster")) {
+      on.exit(parallel::stopCluster(cl))
     }
   }
 
@@ -158,10 +151,6 @@ get_levelpaths <- function(x, override_factor = NULL, status = FALSE, cores = NU
     }
   }
 
-  if(stop_cluster) {
-    parallel::stopCluster(cl)
-  }
-
   outlets <- x %>%
     group_by(.data$levelpath) %>%
     filter(topo_sort == min(topo_sort)) %>%
@@ -312,6 +301,7 @@ get_sorted <- function(x) {
 #' @param x two column data.frame with IDs and toIDs. Names are ignored.
 #' @param outlets IDs of outlet flowlines
 #' @export
+#' @return data.frame containing the terminal ID for each outlet
 #' @importFrom igraph dfs graph_from_data_frame V
 #' @importFrom sf st_drop_geometry
 #' @importFrom tidyr unnest
@@ -362,6 +352,7 @@ get_terminal <- function(x, outlets) {
 #' @importFrom dplyr arrange
 #' @importFrom methods as
 #' @export
+#' @return data.frame containing levelpaths for each ID
 #' @examples
 #' source(system.file("extdata", "walker_data.R", package = "nhdplusTools"))
 #'
@@ -372,12 +363,16 @@ get_terminal <- function(x, outlets) {
 #'
 get_pathlength <- function(x) {
 
+  if(any(x$ID == 0)) stop("ID 0 must not be present. It is used as the outlet ID.")
+
+  x$toID[is.na(x$toID)] <- 0
+
   sorted <- as(get_sorted(x[, c("ID", "toID")]),
                class(x$ID))
 
   sorted <- sorted[length(sorted):1]
 
-  sorted <- sorted[!is.na(sorted)]
+  sorted <- sorted[(!sorted == 0)]
 
   order <- match(sorted, x$ID)
 
@@ -393,7 +388,7 @@ get_pathlength <- function(x) {
   toids <- match(toid, id)
 
   for(i in seq_len(length(id))) {
-    if(!is.na(tid <- toid[i])) {
+    if((tid <- toid[i]) != 0) {
 
       leo[i] <- le[toids[i]] + leo[toids[i]]
 
