@@ -313,6 +313,15 @@ test_that("by rpu", {
 
   expect(nrow(subset_rpu(sample_flines, rpu = "07b")), 267)
   expect(nrow(subset_rpu(sample_flines, rpu = "07b", run_make_standalone = TRUE)), 267)
+
+  suppressWarnings(sample_flines <- dplyr::left_join(
+    dplyr::select(sample_flines, COMID, RPUID, TerminalPa,
+                  ArbolateSu, DnHydroseq, Pathlength, FTYPE),
+    prepare_nhdplus(sample_flines,
+                    0, 0, 0, FALSE), by = "COMID"))
+
+  expect(nrow(subset_rpu(sample_flines, rpu = "07b")), 267)
+  expect(nrow(subset_rpu(sample_flines, rpu = "07b", run_make_standalone = FALSE)), 267)
 })
 
 
